==> avemilter-0.2.tgz: avemilter-0.2/kavcomm.c <==
/*
 * Copyright (c) 2004 Damir Bikmuhametov <boco@ufanet.ru>
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: kavcomm.c,v 1.2 2004/02/22 09:56:28 boco Exp $
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <stdarg.h>

#include "common.h"
#include "kavcomm.h"

static char * extract_virus_name(char *);

/* main scanning procedure */

int kav_scan_file(char * filename, char ** viruslist)
{
	char * id = "kav_scan_file";
	char buffer[4096];
	int fd = -1;
	int len;
	unsigned long rep_len;
	char * p, * q, * r;
	char * virus_names[11];
	char ** virus_name;
	int viruses = 1;

	if (!filename || !*filename) {
		syslog(LOG_ERR, "%s: empty filename", id);
		return -1;
	}

	if (!av_engine_socket || !*av_engine_socket) {
		syslog(LOG_ERR, "%s: empty KAV socket name", id);
		return -1;
	}

	/* connect to KAV daemon */

	if ((fd = av_engine_connect(av_engine_socket)) == -1) {
		syslog(LOG_ERR, "%s: error connecting to KAV socket: %m", id);
		return -1;
	}

	/* prepare query */

	memset(buffer, 0, sizeof(buffer));
	snprintf(buffer, sizeof(buffer) - 35, "<0>Jan  1 00:00:00:\xfeI0|K|O-|s|o{%s}\xfe", filename);
	len = strlen(buffer) + 1;

	/* send query */

	if (timed_write(fd, buffer, len, av_engine_timeout) != len) {
		syslog(LOG_ERR, "%s: can't write to KAV socket: %m", id);
		av_engine_disconnect(fd);
		return -1;
	}

	/* read first part of answer */

	memset(buffer, 0, sizeof(buffer));

	if (timed_read(fd, buffer, 2, av_engine_timeout) != 2) {
		syslog(LOG_ERR, "%s: can't read from KAV socket: %m", id);
		av_engine_disconnect(fd);
		return -1;
	}

	switch (buffer[0] & 0x0F) {
		case 2:		/* corrupted or modified virus found */
		case 3:		/* suspicious object */
		case 4:		/* virus found */
		case 5:		/* virus cured */
		case 6:		/* virus destroyed */
			break;
		case 0:		/* clean object */
			av_engine_disconnect(fd);
			return 0;
		default:
			av_engine_disconnect(fd);
			syslog(LOG_ERR, "%s: can't scan object, KAV return code: 0x%02X%02X", id, buffer[0] & 0xFF, buffer[1] & 0xFF);
			return -1;
	}

	/* here we are with infected object, so check if there is report
	 * from KAV */

	if (!(buffer[1] & 0x01)) {
		av_engine_disconnect(fd);
		return 1;	/* there was a virus */
	}

	/* read in report length */

	rep_len = 0;

	if (timed_read(fd, &rep_len, 4, av_engine_timeout) != 4) {
		syslog(LOG_ERR, "%s: can't read from KAV socket: %m", id);
		av_engine_disconnect(fd);
		return 1;	/* there was a virus */
	}

	if (rep_len >= sizeof(buffer))
		rep_len = sizeof(buffer) - 1;

	memset(buffer, 0, sizeof(buffer));
	p = buffer;

	/* read in report */

	while (rep_len && (len = timed_read(fd, p, 1, av_engine_timeout) > 0)) {

		/* i know it's ugly to read by 1-byte chunks but i can't make
		 * it work with timed_read(fd, p, rep_len, timeout) */

		p += len;
		rep_len -= len;
	}

	if (rep_len || len < 0) {
		syslog(LOG_ERR, "%s: can't read from KAV socket: %m", id);
		av_engine_disconnect(fd);
		return 1;	/* there was a virus */
	}

	av_engine_disconnect(fd);

	/* prepare array for virus names */

	memset(virus_names, 0, sizeof(virus_names));
	virus_name = virus_names;

	/* proceed with report */

	p = buffer;
	while ((q = getline(&p))) {
		if (viruses < sizeof(virus_names)/sizeof(char *)) {

			if ((r = strstr(q, "\tinfected: ")))
				r += 10;
			else if ((r = strstr(q, "\tsuspicion: ")))
				r += 11;
			else if ((r = strstr(q, "    infected: ")))
				r += 13;
			else if ((r = strstr(q, "    suspicion: ")))
				r += 14;

			if (!r || !*r)
				continue;

			*virus_name = extract_virus_name(r);
			if (*virus_name) {
				virus_name++;
				viruses++;
			}
		}
	}

	len = 0;
	virus_name = virus_names;
	while (*virus_name) {
		len += strlen(*virus_name) + 1;
		virus_name++;
	}
	len++;
	if ((*viruslist = (char *) malloc(len))) {
		memset(*viruslist, 0, len);
		virus_name = virus_names;
		while (*virus_name) {
			strcat(*viruslist, *virus_name);
			strcat(*viruslist, " ");
			virus_name++;
		}
	}

	FREE_ARRAY(virus_names);

	return 1;	/* there was a virus */
}

/* this routine gets string returned by AVE and returns virus name */

static char * extract_virus_name(char * line)
{
	int len;
	char * name = NULL;

	/* line points almost to virus name */

	line += strspn(line, "\t ");
	len = strcspn(line, "\n\r\t ");
	line[len] = 0;

	if ((name = (char *) malloc(len + 1))) {
		memset(name, 0, len + 1);
		strncat(name, line, len);
	}

	return name;
}

==> avemilter-0.2.1.tgz: avemilter-0.2.1/kavcomm.c <==
/*
 * Copyright (c) 2004 Damir Bikmuhametov <boco@ufanet.ru>
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: kavcomm.c,v 1.4 2004/03/14 10:41:41 boco Exp $
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <stdarg.h>

#include "common.h"
#include "kavcomm.h"

static char * extract_virus_name(char *);

/* main scanning procedure */

int kav_scan_file(char * filename, char ** viruslist)
{
	char * id = "kav_scan_file";
	char buffer[4096];
	int fd = -1;
	int len;
	unsigned int rep_len;
	char * p, * q, * r;
	char * virus_names[11];
	char ** virus_name;
	int viruses = 1;

	if (!filename || !*filename) {
		syslog(LOG_ERR, "%s: empty filename", id);
		return -1;
	}

	if (!av_engine_socket || !*av_engine_socket) {
		syslog(LOG_ERR, "%s: empty KAV socket name", id);
		return -1;
	}

	/* connect to KAV daemon */

	if ((fd = av_engine_connect(av_engine_socket)) == -1) {
		syslog(LOG_ERR, "%s: error connecting to KAV socket: %m", id);
		return -1;
	}

	/* prepare query */

	memset(buffer, 0, sizeof(buffer));
	snprintf(buffer, sizeof(buffer) - 35, "<0>Jan  1 00:00:00:\xfeI0|K|O-|s|o{%s}\xfe", filename);
	len = strlen(buffer) + 1;

	/* send query */

	if (timed_write(fd, buffer, len, av_engine_timeout) != len) {
		syslog(LOG_ERR, "%s: can't write to KAV socket: %m", id);
		av_engine_disconnect(fd);
		return -1;
	}

	/* read first part of answer */

	memset(buffer, 0, sizeof(buffer));

	if (timed_read(fd, buffer, 2, av_engine_timeout) != 2) {
		syslog(LOG_ERR, "%s: can't read from KAV socket: %m", id);
		av_engine_disconnect(fd);
		return -1;
	}

	switch (buffer[0] & 0x0F) {
		case 2:		/* corrupted or modified virus found */
		case 3:		/* suspicious object */
		case 4:		/* virus found */
		case 5:		/* virus cured */
		case 6:		/* virus destroyed */
			break;
		case 0:		/* clean object */
			av_engine_disconnect(fd);
			return 0;
		default:
			av_engine_disconnect(fd);
			syslog(LOG_ERR, "%s: can't scan object, KAV return code: 0x%02X%02X", id, buffer[1] & 0xFF, buffer[0] & 0xFF);
			return -1;
	}

	/* here we are with infected object, so check if there is report
	 * from KAV */

	if (!(buffer[1] & 0x01)) {
		av_engine_disconnect(fd);
		return 1;	/* there was a virus */
	}

	/* read in report length */

	rep_len = 0;

	if (timed_read(fd, &rep_len, 4, av_engine_timeout) != 4) {
		syslog(LOG_ERR, "%s: can't read from KAV socket: %m", id);
		av_engine_disconnect(fd);
		return 1;	/* there was a virus */
	}

	if (rep_len >= sizeof(buffer))
		rep_len = sizeof(buffer) - 1;

	memset(buffer, 0, sizeof(buffer));
	p = buffer;

	/* read in report */

	while (rep_len && (len = timed_read(fd, p, rep_len, av_engine_timeout)) > 0) {
		p += len;
		rep_len -= len;
	}

	if (rep_len || len < 0) {
		syslog(LOG_ERR, "%s: can't read from KAV socket: %m", id);
		av_engine_disconnect(fd);
		return 1;	/* there was a virus */
	}

	av_engine_disconnect(fd);

	/* prepare array for virus names */

	memset(virus_names, 0, sizeof(virus_names));
	virus_name = virus_names;

	/* proceed with report */

	p = buffer;
	while ((q = getline(&p))) {
		if (viruses < sizeof(virus_names)/sizeof(char *)) {

			if ((r = strstr(q, "\tinfected: ")))
				r += 10;
			else if ((r = strstr(q, "\tsuspicion: ")))
				r += 11;
			else if ((r = strstr(q, "    infected: ")))
				r += 13;
			else if ((r = strstr(q, "    suspicion: ")))
				r += 14;

			if (!r || !*r)
				continue;

			*virus_name = extract_virus_name(r);
			if (*virus_name) {
				virus_name++;
				viruses++;
			}
		}
	}

	len = 0;
	virus_name = virus_names;
	while (*virus_name) {
		len += strlen(*virus_name) + 1;
		virus_name++;
	}
	len++;
	if ((*viruslist = (char *) malloc(len))) {
		memset(*viruslist, 0, len);
		virus_name = virus_names;
		while (*virus_name) {
			strcat(*viruslist, *virus_name);
			strcat(*viruslist, " ");
			virus_name++;
		}
	}

	FREE_ARRAY(virus_names);

	return 1;	/* there was a virus */
}

/* this routine gets string returned by AVE and returns virus name */

static char * extract_virus_name(char * line)
{
	int len;
	char * name = NULL;

	/* line points almost to virus name */

	line += strspn(line, "\t ");
	len = strcspn(line, "\n\r\t ");
	line[len] = 0;

	if ((name = (char *) malloc(len + 1))) {
		memset(name, 0, len + 1);
		strncat(name, line, len);
	}

	return name;
}
